module Coda_Segreteria

implicit none

! Tipico nodo della coda
type nodo
   integer             :: n_prog
   type(nodo), pointer :: prox
end type nodo

! Tipo coda (lista concatenata)
type coda
   type(nodo), pointer :: pt_primo
end type coda

contains

   !***************************************************************************

   function CreaCoda() result(coda_out)
   
   ! Questa implementazione usa solo un puntatore al primo elemento effettivo
   ! della lista concatenata
   implicit none

   type(coda) :: coda_out

   ! Fai puntare a vuoto il puntatore al primo elemento
   coda_out%pt_primo => null()

   end function CreaCoda

   !***************************************************************************

   function METTI_IN_CODA(servizio,coda_R,coda_S,coda_P) result(n_assegnato)

   implicit none

   character(len=1), intent(in) :: servizio
   type(coda), intent(inout)    :: coda_R,coda_S,coda_P
   integer                      :: n_assegnato
   type(nodo), pointer          :: nuovo_nodo,nodo_corrente


   ! Aggiungi subito un nuovo nodo e assegnagli numero 1
   allocate(nuovo_nodo)
   nuovo_nodo%n_prog = 1
   nuovo_nodo%prox => null()
   n_assegnato = 1

   ! Scegli a che servizio sei interessato
   select case(servizio)
      case('R')
         if (.not.associated(coda_R%pt_primo)) then
            ! Se la coda è vuota, aggiungi il primo elemento in testa
            coda_R%pt_primo => nuovo_nodo
            return
         else
            ! Se non è vuota, prendi il primo elemento
            nodo_corrente => coda_R%pt_primo
         endif
      case('S')
         if (.not.associated(coda_S%pt_primo)) then
            ! Se la coda è vuota, aggiungi il primo elemento in testa
            coda_R%pt_primo => nuovo_nodo
            return
         else
            ! Se non è vuota, prendi il primo elemento
            nodo_corrente => coda_S%pt_primo
         endif
      case('P')
         if (.not.associated(coda_P%pt_primo)) then
            ! Se la coda è vuota, aggiungi il primo elemento in testa
            coda_P%pt_primo => nuovo_nodo
            return
         else
            ! Se non è vuota, prendi il primo elemento
            nodo_corrente => coda_P%pt_primo
         endif
      case default
         write(*,*) 'Servizio non disponibile'
         return
   end select

   ! Vai all'ultimo elemento e assegnagli il numero giusto
   do while (associated(nodo_corrente%prox))
      nodo_corrente => nodo_corrente%prox
   enddo
   nodo_corrente%prox => nuovo_nodo
   n_assegnato = nodo_corrente%n_prog + 1
   nuovo_nodo%n_prog = n_assegnato

   end function METTI_IN_CODA

   !***************************************************************************

   function CHIAMA_ALLO_SPORTELLO(sportello,coda_R,coda_S,coda_P) result(n_chiamare)

   implicit none

   character(len=1), intent(in) :: sportello
   type(coda), intent(inout)    :: coda_R,coda_S,coda_P
   integer                      :: n_chiamare
   type(nodo), pointer          :: nodo_chiamare

   ! Scegli quale coda chiamare
   select case(sportello)
      case('R')
         nodo_chiamare => coda_R%pt_primo
         if (.not.associated(nodo_chiamare)) then
            ! Se la coda è vuota, restituisci 0
            n_chiamare = 0
            return
         else
            ! Se non è vuota, metti il secondo elemento come primo
            coda_R%pt_primo => coda_R%pt_primo%prox
         endif
      case('S')
         nodo_chiamare => coda_S%pt_primo
         if (.not.associated(nodo_chiamare)) then
            ! Se la coda è vuota, restituisci 0
            n_chiamare = 0
            return
         else
            ! Se non è vuota, metti il secondo elemento come primo
            coda_S%pt_primo => coda_S%pt_primo%prox
         endif
      case('P')
         nodo_chiamare => coda_P%pt_primo
         if (.not.associated(nodo_chiamare)) then
            ! Se la coda è vuota, restituisci 0
            n_chiamare = 0
            return
         else
            ! Se non è vuota, metti il secondo elemento come primo
            coda_P%pt_primo => coda_P%pt_primo%prox
         endif
      case default
         write(*,*) 'Sportello non disponibile'
         return
   end select

   ! Recupera il numero da chiamare e dealloca il nodo
   n_chiamare = nodo_chiamare%n_prog
   deallocate(nodo_chiamare)
   
   end function CHIAMA_ALLO_SPORTELLO

   !***************************************************************************

end module Coda_Segreteria
