program MetPot
!--------------------------------------------------------------------
! Programma che implementa il metodo delle potenze per la ricerca
! dell'autovalore max
!--------------------------------------------------------------------
implicit none

integer                   :: n,ITMAX
integer                   :: i,j,iter,info
real(kind=8)              :: TOLL
real(kind=8), allocatable :: A(:,:),z_old(:),z_new(:),y(:),res(:)
real(kind=8)              :: lambda,residuo,num,den
real(kind=8)              :: EUCL

! Lettura dati
open(1,file='Dati.in')
read(1,*) n
allocate(A(n,n),z_old(n),z_new(n),y(n),res(n),stat=info)
if (info .ne. 0) then
   write(*,*) 'Errore allocazione'
   stop
endif
do i = 1,n
   read(1,*) (A(i,j),j=1,n)
enddo
do i = 1,n
   read(1,*) z_old(i)
enddo
read(1,*) TOLL
read(1,*) ITMAX
close(1)

! Inizializzazione
residuo = 2.d0*TOLL
iter = 0
do while ((residuo .gt. TOLL) .and. (iter .lt. ITMAX))
   iter = iter + 1
   ! Passo 1
   call MATVET(n,A,z_old,y)
   ! Passo 2
   lambda = EUCL(n,y)
   ! Passo 3
   do i = 1,n
      z_new(i) = y(i) / lambda
   enddo
   ! Calcolo numeratore residuo
   do i = 1,n
      res(i) = y(i) - lambda*z_old(i)
   enddo
   ! Calcolo residuo
   num = EUCL(n,res)
   den = EUCL(n,z_old)
   residuo = num/den
   ! Passo alla prossima iterazione
   do i = 1,n
      z_old(i) = z_new(i)
   enddo
enddo

write(*,*) 'Effettuate iterazioni:',iter
write(*,*)
write(*,*) 'Autovalore:',lambda
write(*,*)
write(*,'(a,e15.6)') 'Autovettore:'
do i = 1,n
   write(*,'(e15.6)') z_old(i)
enddo

! Dealloca tutto
deallocate(A,z_old,z_new,y,res,stat=info)
if (info .ne. 0) then
   write(*,*) 'Errore deallocazione'
   stop
endif

end program MetPot

!*********************************************************************

subroutine MATVET(n,A,x,y)

implicit none

integer, intent(in)       :: n
real(kind=8), intent(in)  :: A(n,n),x(n)
real(kind=8), intent(out) :: y(n)
integer                   :: i,j

do i = 1,n
   y(i) = 0.d0
   do j =1,n
      y(i) = y(i) + A(i,j)*x(j)
   enddo
enddo

end subroutine MATVET

!*********************************************************************

function EUCL(n,v) result(norma)

implicit none

integer, intent(in)      :: n
real(kind=8), intent(in) :: v(n)
real(kind=8)             :: norma
integer                  :: i

norma = 0.d0
do i = 1,n
   norma = norma + v(i)**2
enddo
norma = sqrt(norma)

end function EUCL
